package cz.czu.pef.spaceboy

import android.app.AlarmManager
import android.app.PendingIntent
import android.content.Context
import android.content.Context.MODE_PRIVATE
import android.content.Intent
import java.util.*

class AlrManager {

    private var alarmMgr: AlarmManager? = null
    lateinit var context: Context

    /**
     * Nastavení hodin notifikace
     */
    var hourApod = 9
    var hourAsteroids = 12
    var hourIss = 17

    /**
     * Metoda nastaví alarmy pro všechny 3 typy notifikací
     */
    fun setAlarm(cont: Context) {

        context = cont
        alarmMgr = context.getSystemService(Context.ALARM_SERVICE) as AlarmManager
        val alarmIntent = Intent(context, NotificationReceiver::class.java)

        val prefs = context.getSharedPreferences("SharedPreferences", MODE_PRIVATE)

        /**
         * Na základě nastavení v SharedPreferences se zavolají metody pro spuštění alarmů
         */
        if (prefs.getBoolean("notifApod", true)) {
            startAlarm(alarmIntent, 1, hourApod)}
        else {cancelAlarm(alarmIntent, 1)}

        if (prefs.getBoolean("notifAsteroids", true)) {
            startAlarm(alarmIntent, 2, hourAsteroids)}
        else {cancelAlarm(alarmIntent, 2)}

        if (prefs.getBoolean("notifIss", true)) {
            startAlarm(alarmIntent, 3, hourIss)}
        else {cancelAlarm(alarmIntent, 3)}
    }

    /**
     * Metoda nastaví pouze vyžádaný typ alarmu pro konkrétní typ notifikace
     */
    fun setAlarmSingle(cont: Context, key: String, type: Int) {

        context = cont
        alarmMgr = context.getSystemService(Context.ALARM_SERVICE) as AlarmManager
        val alarmIntent = Intent(context, NotificationReceiver::class.java)

        val prefs = context.getSharedPreferences("SharedPreferences", MODE_PRIVATE)

        if (prefs.getBoolean(key, true)) {
            when (type) {
                1 -> startAlarm(alarmIntent, type, hourApod)
                2 -> startAlarm(alarmIntent, type, hourAsteroids)
                3 -> startAlarm(alarmIntent, type, hourIss)
            }
        }
        else {cancelAlarm(alarmIntent, type)}
    }

    /**
     * Metoda provádějící samotné založení a spuštění alarmu
     */
    private fun startAlarm(alarmIntent: Intent, type: Int, hour: Int) {

        alarmIntent.putExtra("type", type)

        val pendingIntent = PendingIntent.getBroadcast(context, type, alarmIntent, PendingIntent.FLAG_UPDATE_CURRENT)

        val calendar = Calendar.getInstance().apply {
            timeInMillis = System.currentTimeMillis()
            set(Calendar.HOUR_OF_DAY, hour)}

        if (Calendar.getInstance().after(calendar)) {
            calendar.add(Calendar.DAY_OF_MONTH, 1)
        }

        alarmMgr!!.setInexactRepeating(
            AlarmManager.RTC_WAKEUP,
            calendar.timeInMillis,
            AlarmManager.INTERVAL_DAY,
            pendingIntent)
    }

    /**
     * Metoda ukončující běžící alarm
     */
    private fun cancelAlarm(alarmIntent: Intent, type: Int) {

        val pendingIntent = PendingIntent.getBroadcast(context, type, alarmIntent, PendingIntent.FLAG_UPDATE_CURRENT)

        alarmMgr?.cancel(pendingIntent)
    }
}