package cz.czu.pef.spaceboy

import android.content.Intent
import android.net.Uri
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.view.View
import android.widget.Toast
import com.google.gson.GsonBuilder
import com.squareup.picasso.Picasso
import kotlinx.android.synthetic.main.activity_apod.*
import okhttp3.*
import java.io.IOException
import java.util.*


class ApodActivity : AppCompatActivity() {

    val apiKey = "J5HWLtcA9ZgTxUu7bAfjL3xcSm0gLeHI2JMk8JMI"

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_apod)

        val connectivityCheck = ConnectivityCheck()
        if (connectivityCheck.isConnected(this)) {
            fetchJson()
        }
        else {
            Toast.makeText(this, R.string.no_connection, Toast.LENGTH_LONG).show()
        }
    }


    private fun fetchJson() {
        val url = "https://api.nasa.gov/planetary/apod?api_key=$apiKey"
        val request = Request.Builder().url(url).build()
        val client = OkHttpClient()

        client.newCall(request).enqueue(object: Callback {
            override fun onResponse(call: Call, response: Response) {
                val body = response.body()?.string()
                val gson = GsonBuilder().create()
                val apod = gson.fromJson(body, AstronomyPicture::class.java)

                runOnUiThread {
                    textApod_title.text = apod.title

                    when (apod.media_type) {
                        "image" -> {
                            Picasso.get().load(apod.url).into(imageApod_apod)
                            buttonApod_hdUrl.visibility = View.VISIBLE

                            buttonApod_hdUrl.setOnClickListener {
                                startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(apod.hdurl)))
                            }
                        }
                        "video" -> {
                            Picasso.get().load(R.drawable.logo_youtube).into(imageApod_apod)

                            imageApod_apod.setOnClickListener {
                                startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(apod.url)))
                            }
                        }
                    }

                    textApod_explanation.text = apod.explanation

                    textApod_date.text = (formatDate(apod.date))
                    labelApod_date.visibility = View.VISIBLE

                    if(apod?.copyright != null) {
                        textApod_copyright.text = apod.copyright
                        textApod_copyright.visibility = View.VISIBLE
                        labelApod_copyright.visibility = View.VISIBLE
                    }
                }
            }

            override fun onFailure(call: Call, e: IOException) {
                println("Failed to execute request")
            }
        })
    }

    /**
     * Zobrazení datumu, s odpovídajícím formátem dle lokalizace zařízení
     * Zde se z API vrací datum ve formě Stringu, takže formátování jako
     * DATE objektu nelze
     *
     * CZ formát data "dd.MM.yyyy"
     * EN formát data "MMM dd, yyyy"
     *
     * Přetypování stringu na object date lze snadno dosáhnout pomocí metody,
     * ta je ovšem dostupná s minimal API 26, v tomto projektu je minimal 21,
     * z toho důvodu je napsán vlastní metoda formatDate()
     */

    fun formatDate(strDate: String): String {
        val dateArray = strDate.split("-").toTypedArray()
        val formattedDate: String
        if (Locale.getDefault().displayLanguage == "čeština") {
            formattedDate = "${dateArray[2]}.${dateArray[1]}.${dateArray[0]}"
        }
        else {
            var month = ""
            when(dateArray[1]) {
                "01" -> month = "Jan"
                "02" -> month = "Feb"
                "03" -> month = "Mar"
                "04" -> month = "Apr"
                "05" -> month = "May"
                "06" -> month = "Jun"
                "07" -> month = "Jul"
                "08" -> month = "Aug"
                "09" -> month = "Sep"
                "10" -> month = "Oct"
                "11" -> month = "Nov"
                "12" -> month = "Dec"
            }
            formattedDate = "$month ${dateArray[2]}, ${dateArray[0]}"
        }
        return formattedDate
    }
}

data class AstronomyPicture(val copyright: String,
                       val date: String,
                       val explanation: String,
                       val media_type: String,
                       val hdurl: String,
                       val title: String,
                       val url: String)