package cz.czu.pef.spaceboy

import android.content.Intent
import android.net.Uri
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import com.google.gson.GsonBuilder
import kotlinx.android.synthetic.main.activity_asteroids.*
import okhttp3.*
import java.io.IOException
import java.text.SimpleDateFormat
import java.util.*
import com.squareup.picasso.Picasso
import kotlinx.android.synthetic.main.asteroid_row.view.*
import java.text.NumberFormat

class AsteroidsActivity : AppCompatActivity() {

    val apikey = "J5HWLtcA9ZgTxUu7bAfjL3xcSm0gLeHI2JMk8JMI"

    /**
    COMPANION OBJECT slouží pro předání přístupu do R.STRING jiné třídě (IssPassesAdapter)
     */
    companion object {
        lateinit var instance: AsteroidsActivity
        private set
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_asteroids)

        val connectivityCheck = ConnectivityCheck()
        if (connectivityCheck.isConnected(this)) {
            instance = this
            recyclerView_asteroids.layoutManager = androidx.recyclerview.widget.LinearLayoutManager(this)
            fetchJson()
        }
        else {
            Toast.makeText(this, R.string.no_connection, Toast.LENGTH_LONG).show()
        }
    }

    fun fetchJson() {

        val today = SimpleDateFormat("yyyy-MM-dd", Locale.US).format(Calendar.getInstance().time)
        val url = "https://api.nasa.gov/neo/rest/v1/feed?end_date=$today&api_key=$apikey"

        val request = Request.Builder().url(url).build()

        val client = OkHttpClient()
        client.newCall(request).enqueue(object: Callback {
            override fun onResponse(call: Call, response: Response) {
                val body = response.body()?.string()

                val gson = GsonBuilder().create()

                val asteroidsFeed = gson.fromJson(body, AsteroidsFeed::class.java)

                runOnUiThread {
                    recyclerView_asteroids.adapter = AsteroidAdapter(asteroidsFeed)
                }
            }

            override fun onFailure(call: Call, e: IOException) {
                println("Failed to execute request")
            }
        })
    }
}

data class AsteroidsFeed(val element_count: Int,
                         val near_earth_objects: Map<String, Array<Asteroids>>)

data class Asteroids(val name: String,
                     val nasa_jpl_url: String,
                     val estimated_diameter: EstimatedDiameter,
                     val close_approach_data: Array<CloseApproachData>)

data class EstimatedDiameter(val meters: Meters)

data class Meters(val estimated_diameter_min: Double,
                  val estimated_diameter_max: Double)

data class CloseApproachData(val epoch_date_close_approach: Long,
                             val relative_velocity: RelativeVelocity,
                             val miss_distance: MissDistance)

data class RelativeVelocity(val kilometers_per_hour: String)

data class MissDistance(val kilometers: String)

class AsteroidAdapter(val asteroids: AsteroidsFeed): androidx.recyclerview.widget.RecyclerView.Adapter<AsteroidViewHolder>() {

    private val neo = asteroids.near_earth_objects.toList().first().second

    override fun getItemCount(): Int {
        return asteroids.element_count
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): AsteroidViewHolder {
        val layoutInflater = LayoutInflater.from(parent.context)
        val cellForRow = layoutInflater.inflate(R.layout.asteroid_row, parent, false)
        return AsteroidViewHolder(cellForRow)
    }

    override fun onBindViewHolder(holder: AsteroidViewHolder, position: Int) {

        val passes = neo[position]

        val instance = AsteroidsActivity.instance

        val sdfEn = SimpleDateFormat("MMM dd, yyyy HH:mm", Locale.US)
        val sdfCz = SimpleDateFormat("dd.MM.yyyy HH:mm", Locale.FRANCE)
        val numEn = NumberFormat.getInstance(Locale.US)
        val numCz = NumberFormat.getInstance(Locale.FRANCE)

        val distance: String
        val velocity: String
        val meanDia = ((passes.estimated_diameter.meters.estimated_diameter_min +
                passes.estimated_diameter.meters.estimated_diameter_max) / 2).toInt()

        if(Locale.getDefault().displayLanguage == "čeština") {
            holder.view.text_date.text = sdfCz.format(passes.close_approach_data[0].epoch_date_close_approach)
            distance = numCz.format((passes.close_approach_data[0].miss_distance.kilometers.toDouble()).toInt()) + " km"
            velocity = numCz.format(((passes.close_approach_data[0].relative_velocity.kilometers_per_hour).toDouble()).toInt()) + " ${instance.getString(R.string.km_p_h)}"
        }
        else {
            holder.view.text_date.text = sdfEn.format(passes.close_approach_data[0].epoch_date_close_approach)
            distance = numEn.format((passes.close_approach_data[0].miss_distance.kilometers.toDouble()).toInt()) + " km"
            velocity = numEn.format(((passes.close_approach_data[0].relative_velocity.kilometers_per_hour).toDouble()).toInt()) + " ${instance.getString(R.string.km_p_h)}"
        }

        val diameter = "${passes.estimated_diameter.meters.estimated_diameter_min.toInt()} - " +
                "${passes.estimated_diameter.meters.estimated_diameter_max.toInt()} m"


        if (meanDia < 100) {
            Picasso.get().load(R.drawable.asteroid_1).fit().into(holder.view.imageView_asteroid)
        }
        else if (meanDia < 500) {
            Picasso.get().load(R.drawable.asteroid_2).fit().into(holder.view.imageView_asteroid)
        }
        else if (meanDia < 1500) {
            Picasso.get().load(R.drawable.asteroid_3).fit().into(holder.view.imageView_asteroid)
        }
        else {
            Picasso.get().load(R.drawable.asteroid_4).fit().into(holder.view.imageView_asteroid)
        }

        holder.view.text_asteroidName.text = passes.name
        holder.view.text_estimatedDiameter.text = diameter
        holder.view.text_approachDistance.text = distance
        holder.view.text_velocity.text = velocity

        /**
         * Slouží pro předání parametru URL adresy třídě AsteroidViewHolder
         */
        holder.url = passes.nasa_jpl_url
    }
}


class AsteroidViewHolder(val view: View, var url: String? = null): androidx.recyclerview.widget.RecyclerView.ViewHolder(view) {

    init {
        view.imageView_asteroid_info.setOnClickListener {
            view.context.startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(url!!)))
        }
    }
}