package cz.czu.pef.spaceboy

import android.Manifest.permission.ACCESS_COARSE_LOCATION
import android.content.Intent
import android.content.pm.PackageManager
import android.content.pm.PackageManager.PERMISSION_GRANTED
import android.net.Uri
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.provider.Settings
import android.view.View
import android.widget.Toast
import androidx.core.app.ActivityCompat
import com.google.android.material.snackbar.Snackbar
import com.google.android.material.snackbar.Snackbar.LENGTH_INDEFINITE
import com.squareup.picasso.Picasso
import cz.czu.pef.spaceboy.BuildConfig.APPLICATION_ID
import kotlinx.android.synthetic.main.activity_iss.*

class IssActivity : AppCompatActivity() {

    private val REQUEST_PERMISSIONS_REQUEST_CODE = 34

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_iss)

        /**
         * Přiřazení obrázků do položek podmenu.
         */
        runOnUiThread {
            Picasso.get().load(R.drawable.iss_position).fit().centerCrop().into(image_issPosition)
            Picasso.get().load(R.drawable.iss_passes).fit().centerCrop().into(image_issPasses)
            Picasso.get().load(R.drawable.iss_astronauts).fit().centerCrop().into(image_issAstronauts)
        }

        val connectivityCheck = ConnectivityCheck()

        cardView_IssPosition.setOnClickListener{
            if (connectivityCheck.isConnected(this)) {
                startActivity(Intent (this, IssPositionActivity::class.java))
            }
            else {
                Toast.makeText(this, R.string.no_connection, Toast.LENGTH_LONG).show()
            }
        }

        /**
         * Metoda setOnClickListener spouští aktivitu IssPassesActivity,
         * k jejíž funkci je potřeba povolení lokace v zařízení.
         * Pokud lokace povolená není, inicializuje se metoda requestPermissions(),
         * která po uživateli vyžádá povolení.
         */
        cardView_IssPasses.setOnClickListener {
            if (connectivityCheck.isConnected(this)) {
                if (!checkPermissions()) {
                    requestPermissions()
                }
                else {
                    startActivity(Intent(this, IssPassesActivity::class.java))
                }
            }
            else {
                Toast.makeText(this, R.string.no_connection, Toast.LENGTH_LONG).show()
            }
        }

        cardView_IssAstronauts.setOnClickListener {
            if (connectivityCheck.isConnected(this)) {
                startActivity(Intent (this, IssAstronautsActivity::class.java))
            }
            else {
                Toast.makeText(this, R.string.no_connection, Toast.LENGTH_LONG).show()
            }
        }

    }

    /**
     * Metoda ověří povolení lokace v zařízení. Vrací boolean true pokud je lokace povolena.
     */
    private fun checkPermissions() = ActivityCompat.checkSelfPermission(
        this, ACCESS_COARSE_LOCATION) == PERMISSION_GRANTED

    /**
     * Metoda provede samotné vyžádání.
     */
    private fun startLocationPermissionRequest() {
        ActivityCompat.requestPermissions(this, arrayOf(ACCESS_COARSE_LOCATION), REQUEST_PERMISSIONS_REQUEST_CODE)
    }

    /**
     * Metoda inicializuje vyžádání lokace.
     */
    private fun requestPermissions() {
        if (ActivityCompat.shouldShowRequestPermissionRationale(this, ACCESS_COARSE_LOCATION)) {
            showSnackbar(R.string.permission_rationale, android.R.string.ok, View.OnClickListener {
                startLocationPermissionRequest()
            })
        }
        else {
            startLocationPermissionRequest()
        }
    }

    /**
     * Metoda definuje akci po povolení či odmítnutí lokace.
     * V případě povolení spustí požadovanou aktivitu IssPassesActivity,
     * v opačném případě zobrazí chybovou hlášku ve snackbaru.
     */
    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>,
        grantResults: IntArray
    ) {
        if (requestCode == REQUEST_PERMISSIONS_REQUEST_CODE) {
            when {
                (grantResults[0] == PackageManager.PERMISSION_GRANTED) -> startActivity(Intent(this, IssPassesActivity::class.java))

                else -> {
                    showSnackbar(R.string.permission_denied_explanation, R.string.settings,
                        View.OnClickListener {
                            val intent = Intent().apply {
                                action = Settings.ACTION_APPLICATION_DETAILS_SETTINGS
                                data = Uri.fromParts("package", APPLICATION_ID, null)
                                flags = Intent.FLAG_ACTIVITY_NEW_TASK
                            }
                            startActivity(intent)
                        })
                }
            }
        }
    }

    /**
     * Metoda zobrazí snackbar se zprávou.
     */
    private fun showSnackbar(
        snackStrId: Int,
        actionStrId: Int = 0,
        listener: View.OnClickListener? = null
    ) {
        val snackbar = Snackbar.make(findViewById(android.R.id.content), getString(snackStrId), LENGTH_INDEFINITE)
        if (actionStrId != 0 && listener != null) {
            snackbar.setAction(getString(actionStrId), listener)
        }
        snackbar.show()
    }
}
