package cz.czu.pef.spaceboy

import android.content.Intent
import android.net.Uri
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.google.gson.GsonBuilder
import kotlinx.android.synthetic.main.activity_iss_astronauts.*
import kotlinx.android.synthetic.main.iss_astronaut_row.view.*
import okhttp3.*
import java.io.IOException

const val wikiUrl = "https://en.wikipedia.org/wiki/"
const val url = "http://api.open-notify.org/astros.json"

class IssAstronautsActivity : AppCompatActivity() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_iss_astronauts)

        recyclerView_issAstronauts.layoutManager = androidx.recyclerview.widget.LinearLayoutManager(this)

        /**
         * Sestavení URL volání, jehož výsledkem je naplnění response v JSON formátu do datové
         * třídy IssAstronauts
         */
        val request = Request.Builder().url(url).build()
        val client = OkHttpClient()
        client.newCall(request).enqueue(object:Callback {
            override fun onResponse(call: Call, response: Response) {
                val body = response.body()?.string()
                val gson = GsonBuilder().create()
                val issAstr = gson.fromJson(body, IssAstronauts::class.java)

                runOnUiThread {
                    recyclerView_issAstronauts.adapter = IssAstronautsAdapter(issAstr)
                }
            }

            override fun onFailure(call: Call, e: IOException) {
                println("Failed to execute request")
            }
        })
    }
}

data class IssAstronauts(val people: Array<People>, val number: Int)

data class People(val name: String)

/**
 * Třída ovládá obsah uvnitř RecyclerView prvku třemi metodami.
 * getItemCount() ...nastavuje počet prvků v seznamu
 * onCreateViewHolder() ...vytvoří list
 * onBindViewHolder() ...přiřadí informace k jednotlivým views v seznamu
 */
class IssAstronautsAdapter(private val issAstr:IssAstronauts): androidx.recyclerview.widget.RecyclerView.Adapter<AstronautViewHolder>() {

    override fun getItemCount(): Int {
        return issAstr.number
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): AstronautViewHolder {
        val layoutInflater = LayoutInflater.from(parent.context)
        val cellForRow = layoutInflater.inflate(R.layout.iss_astronaut_row, parent, false)
        return AstronautViewHolder(cellForRow)
    }

    override fun onBindViewHolder(holder: AstronautViewHolder, position: Int) {

        val people = issAstr.people[position]
        holder.view.text_astronautName.text = people.name

        /**
         * Předání parametru URL adresy třídě AstronautViewHolder
         */
        holder.astronaut = people
    }
}

class AstronautViewHolder(val view: View, var astronaut: People? = null): androidx.recyclerview.widget.RecyclerView.ViewHolder(view) {

    /**
     * Nastavení prokliku každého view v rámci RecyclerView na otevření konkrétní URL v browseru
     */
    init {
        view.cardView_issAstronaut.setOnClickListener {
            val intent = Intent(Intent.ACTION_VIEW, Uri.parse(wikiUrl + astronaut!!.name))

            view.context.startActivity(intent)
        }
    }
}