package cz.czu.pef.spaceboy

import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import com.google.android.gms.location.FusedLocationProviderClient
import com.google.android.gms.location.LocationServices
import com.google.gson.GsonBuilder
import com.squareup.picasso.Picasso
import kotlinx.android.synthetic.main.activity_iss_passes.*
import kotlinx.android.synthetic.main.iss_pass_row.view.*
import okhttp3.*
import java.io.IOException
import java.text.DateFormat
import java.text.SimpleDateFormat
import java.util.*


class IssPassesActivity : AppCompatActivity() {

    val apiKey = "55UTLD-T7687N-NMAHKB-3YGU"

    /**
    COMPANION OBJECT slouží pro předání přístupu do R.STRING jiné třídě (IssPassesAdapter)
     */
    companion object {
        lateinit var instance: IssPassesActivity
        private set
    }

    private lateinit var fusedLocationClient: FusedLocationProviderClient

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_iss_passes)

        val connectivityCheck = ConnectivityCheck()
        if (connectivityCheck.isConnected(this)) {
            fusedLocationClient = LocationServices.getFusedLocationProviderClient(this)

            getLastLocation()

            instance = this

            recyclerView_issPasses.layoutManager = androidx.recyclerview.widget.LinearLayoutManager(this)
        }
        else {
            Toast.makeText(this, R.string.no_connection, Toast.LENGTH_LONG).show()
        }
    }

    /**
     * Metoda převezme argumenty z metody getLastLocation(), kterými jsou souřadnice polohy zařízení
     * a pomocí nich vytvoří adresu requestu a poté samotný požadavek zašle API
     */
    private fun createRequest(locationSuccess: Boolean, lat: Double, lon: Double, alt: Double) {

        val url: String

        if (locationSuccess) {
            if (alt == 0.0) {
                url = "http://www.n2yo.com/rest/v1/satellite/visualpasses/25544/$lat/$lon/400.0/10/10/?apiKey=$apiKey"
            }
            else {
                url = "http://www.n2yo.com/rest/v1/satellite/visualpasses/25544/$lat/$lon/$alt/10/10/?apiKey=$apiKey"
            }
        }
        else {
            url = "http://www.n2yo.com/rest/v1/satellite/visualpasses/25544/50.08/14.43/400.0/10/10/?apiKey=$apiKey"
        }


        /**
         * Sestavení URL volání, jehož výsledkem je naplnění response v JSON formátu do datové
         * třídy IssPasses
         */
        val request = Request.Builder().url(url).build()
        val client = OkHttpClient()

        client.newCall(request).enqueue(object:Callback {
            override fun onResponse(call: Call, response: Response) {
                val body = response.body()?.string()
                val gson = GsonBuilder().create()
                val issPass = gson.fromJson(body, IssPasses::class.java)

                runOnUiThread {
                    recyclerView_issPasses.adapter = IssPassesAdapter(issPass)
                }
            }

            override fun onFailure(call: Call, e: IOException) {
                println("Failed to execute request")
            }
        })
    }

    /**
     * Po onCreate() je zjištěna lokace metodou getLastLocation(), kteár následně předá informace
     * o lokaci funkci createRequest()
     */
    private fun getLastLocation(){
        fusedLocationClient.lastLocation.addOnCompleteListener(this) { task ->
            if (task.isSuccessful && task.result != null) {
                Toast.makeText(this, R.string.location_detected, Toast.LENGTH_LONG).show()
            }
            else {
                Toast.makeText(this, R.string.no_location_detected, Toast.LENGTH_LONG).show()
            }
            createRequest(task.isSuccessful, task.result!!.latitude, task.result!!.longitude, task.result!!.altitude)
        }
    }
}

/**
 * Data jsou uložena do datových tříd
 */
data class IssPasses(val info: Info,
                val passes: Array<Passes>)

data class Info(val passescount: Int)

data class Passes(
    val startAzCompass: String,
    val maxUTC: Long,
    val endAzCompass: String,
    val mag: Float,
    val duration: Int)

/**
 * Třída obsluhuje nafukovací seznam RecyclerView
 */
class IssPassesAdapter(val issPass:IssPasses): androidx.recyclerview.widget.RecyclerView.Adapter<PassViewHolder>() {

    override fun getItemCount(): Int {
        return issPass.info.passescount
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): PassViewHolder {
        val layoutInflater = LayoutInflater.from(parent.context)
        val cellforRow = layoutInflater.inflate(R.layout.iss_pass_row, parent, false)
        return PassViewHolder(cellforRow)
    }

    override fun onBindViewHolder(holder: PassViewHolder, position: Int) {
        val passes = issPass.passes[position]

        /**
         * Vypsání data přeletu
         */

        /**
         * passDateTime - timestamp vyjádřený v ms - parametr passes.maxUTC se z API vrací
         * ve vteřinách, před transformací do data je potřeba převést do milisekund,
         * čeho se dosáhne vynásobením 1000
         */

        val dateFormatterEn = DateFormat.getDateInstance(DateFormat.DEFAULT, Locale.US)
        val dateFormatterCz = DateFormat.getDateInstance(DateFormat.DEFAULT, Locale.GERMANY)

        /**
         * Pokud uživatel používá na zařízení Češtinu,
         * datum se zformátuje do formátu, běžného v CZ.
         * V opačném případě se zobrazí EN formát data
         */

        if(Locale.getDefault().displayLanguage == "čeština") {
            holder.view.textView_passDate.text = dateFormatterCz.format(passes.maxUTC * 1000)
        }
        else {
            holder.view.textView_passDate.text = dateFormatterEn.format(passes.maxUTC * 1000)
        }

        /**
         * Vypsání časů přeletů
         */
        val sdfTime = SimpleDateFormat("HH:mm:ss", Locale.GERMAN)
        holder.view.textView_passTime.text = sdfTime.format(passes.maxUTC * 1000)

        /**
         * Vypsání směru letu ISS pomocí světových stran s překladem
         */
        val instance = IssPassesActivity.instance
        var dirStart = ""
        var dirEnd = ""
        when(passes.startAzCompass) {
            "N" -> dirStart = instance.getString(R.string.n)
            "NNE" -> dirStart = instance.getString(R.string.nne)
            "NE" -> dirStart = instance.getString(R.string.ne)
            "ENE" -> dirStart = instance.getString(R.string.ene)
            "E" -> dirStart = instance.getString(R.string.e)
            "ESE" -> dirStart = instance.getString(R.string.ese)
            "SE" -> dirStart = instance.getString(R.string.se)
            "SSE" -> dirStart = instance.getString(R.string.sse)
            "S" -> dirStart = instance.getString(R.string.s)
            "SSW" -> dirStart = instance.getString(R.string.ssw)
            "SW" -> dirStart = instance.getString(R.string.sw)
            "WSW" -> dirStart = instance.getString(R.string.wsw)
            "W" -> dirStart = instance.getString(R.string.w)
            "WNW" -> dirStart = instance.getString(R.string.wnw)
            "NW" -> dirStart = instance.getString(R.string.nw)
            "NNW" -> dirStart = instance.getString(R.string.nnw)
        }
        when(passes.endAzCompass) {
            "N" -> dirEnd = instance.getString(R.string.n)
            "NNE" -> dirEnd = instance.getString(R.string.nne)
            "NE" -> dirEnd = instance.getString(R.string.ne)
            "ENE" -> dirEnd = instance.getString(R.string.ene)
            "E" -> dirEnd = instance.getString(R.string.e)
            "ESE" -> dirEnd = instance.getString(R.string.ese)
            "SE" -> dirEnd = instance.getString(R.string.se)
            "SSE" -> dirEnd = instance.getString(R.string.sse)
            "S" -> dirEnd = instance.getString(R.string.s)
            "SSW" -> dirEnd = instance.getString(R.string.ssw)
            "SW" -> dirEnd = instance.getString(R.string.sw)
            "WSW" -> dirEnd = instance.getString(R.string.wsw)
            "W" -> dirEnd = instance.getString(R.string.w)
            "WNW" -> dirEnd = instance.getString(R.string.wnw)
            "NW" -> dirEnd = instance.getString(R.string.nw)
            "NNW" -> dirEnd = instance.getString(R.string.nnw)
        }
        val trajectory = "$dirStart → $dirEnd"
        holder.view.textView_passDirection.text = trajectory

        val duration = "~${passes.duration}s"
        holder.view.textView_passDuration.text = duration

        /**
         * Výběr obrázku, naznačující velikost tělesa
         */
        if (passes.mag <= -2 ) {
            Picasso.get().load(R.drawable.visibility_4).fit().into(holder.view.imageView_passMagnitude)
        }
        else if (passes.mag <= -1) {
            Picasso.get().load(R.drawable.visibility_3).fit().into(holder.view.imageView_passMagnitude)
        }
        else if (passes.mag <= 0) {
            Picasso.get().load(R.drawable.visibility_2).fit().into(holder.view.imageView_passMagnitude)
        }
        else {
            Picasso.get().load(R.drawable.visibility_1).fit().into(holder.view.imageView_passMagnitude)
        }
    }
}

class PassViewHolder(val view: View): androidx.recyclerview.widget.RecyclerView.ViewHolder(view)
