package cz.czu.pef.spaceboy

import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.os.Handler
import com.google.android.gms.maps.CameraUpdateFactory
import com.google.android.gms.maps.GoogleMap
import com.google.android.gms.maps.OnMapReadyCallback
import com.google.android.gms.maps.SupportMapFragment
import com.google.android.gms.maps.model.BitmapDescriptorFactory
import com.google.android.gms.maps.model.LatLng
import com.google.android.gms.maps.model.MarkerOptions
import com.google.gson.GsonBuilder
import okhttp3.*
import java.io.IOException

class IssPositionActivity : AppCompatActivity(), OnMapReadyCallback {

    private lateinit var mMap: GoogleMap

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_iss_position)
        val mapFragment = supportFragmentManager.findFragmentById(R.id.map) as SupportMapFragment
        mapFragment.getMapAsync(this)
    }

    /**
     * Prvotní nastavení ukazatele na nulových souřadnicích
     */
    override fun onMapReady(googleMap: GoogleMap) {
        mMap = googleMap
        val handler = Handler()
        val delay = 1000
        var iss = mMap.addMarker(MarkerOptions().position(LatLng(0.0, 0.0)).visible(false))
        var lat = 0.0
        var lon = 0.0
        val url = "http://api.open-notify.org/iss-now.json"
        val request = Request.Builder().url(url).build()
        val client = OkHttpClient()

        handler.postDelayed(object: Runnable {
            override fun run() {

                /**
                 * Opakující se událost, ve které je požádáno API o nové souřadnice,
                 * na které se následně přenastaví ukazatel každou 1 vteřinu.
                 */
                client.newCall(request).enqueue(object: Callback {
                    override fun onResponse(call: Call, response: Response) {
                        val body = response.body()?.string()
                        val gson = GsonBuilder().create()
                        val issPos = gson.fromJson(body, IssPosition::class.java)
                        lon = issPos.iss_position.longitude.toDouble()
                        lat = issPos.iss_position.latitude.toDouble()
                    }
                    override fun onFailure(call: Call, e: IOException) {
                        println("Failed to execute request")
                    }
                })

                /**
                 * Odstranění stávajícího ukazatele a vytvoření nového na aktuálních souřadnicích
                 */
                iss.remove()
                iss = mMap.addMarker(MarkerOptions().position(LatLng(lat, lon)).icon(BitmapDescriptorFactory.fromResource(R.drawable.map_marker)))
                mMap.moveCamera(CameraUpdateFactory.newLatLng(LatLng(lat, lon)))

                handler.postDelayed(this, delay.toLong())
            }
        }, delay.toLong())
    }
}

data class IssPosition(val iss_position: IssCoordinates)

data class IssCoordinates(val longitude: String, val latitude: String)
