package cz.czu.pef.spaceboy

import android.content.Context
import android.content.Intent
import android.net.Uri
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.widget.Toast
import com.squareup.picasso.Picasso
import kotlinx.android.synthetic.main.activity_main.*

class MainActivity : AppCompatActivity() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        /**
         * Nastavení obrázků do položek menu.
         */
        runOnUiThread {
            Picasso.get().load(R.drawable.menu_apod).fit().centerCrop().into(imageMenuApod)
            Picasso.get().load(R.drawable.menu_asteroids).fit().centerCrop().into(imageMenuAsteroids)
            Picasso.get().load(R.drawable.menu_iss).fit().centerCrop().into(imageMenuIss)
            Picasso.get().load(R.drawable.menu_nasatv).fit().centerCrop().into(imageMenuNasaTv)
            Picasso.get().load(R.drawable.menu_settings).fit().centerCrop().into(imageMenuSettings)
            Picasso.get().load(R.drawable.menu_feedback).fit().centerCrop().into(imageMenuFeedback)
        }

        /**
         * Metody setOnClickListener spouští další aktivity aplikace,
         * některé z nich předtím prověří, zda je zařízení připojené k síti,
         * pokud to funkce, na kterou se přechází, požaduje.
         */
        val connectivityCheck = ConnectivityCheck()

        cardApod.setOnClickListener {
            if (connectivityCheck.isConnected(this)) {
                startActivity(Intent(this, ApodActivity::class.java))
            }
            else {
                Toast.makeText(this, R.string.no_connection, Toast.LENGTH_LONG).show()
            }
        }

        cardAsteroids.setOnClickListener {
            if (connectivityCheck.isConnected(this)) {
                startActivity(Intent (this, AsteroidsActivity::class.java))
            }
            else {
                Toast.makeText(this, R.string.no_connection, Toast.LENGTH_LONG).show()
            }
        }

        cardIss.setOnClickListener {
            startActivity(Intent (this, IssActivity::class.java))
        }

        cardNasaTv.setOnClickListener {
            if (connectivityCheck.isConnected(this)) {
                startActivity(Intent(this, NasaTvActivity::class.java))
            }
            else {
                Toast.makeText(this, R.string.no_connection, Toast.LENGTH_LONG).show()
            }
        }

        cardSettings.setOnClickListener {
            startActivity(Intent(this, SettingsActivity::class.java))
        }

        /**
         * Po tapnutí na položku Zpětná vazba v hlavním menu se spustí výchozí aplikace
         * emailového klienta.
         */
        cardFeedback.setOnClickListener {
            val emailIntent = Intent(Intent.ACTION_VIEW)
            val data : Uri = Uri.parse("mailto:?subject=" + "Feedback" + "&to=" + "spacenetic@post.cz")
            emailIntent.data = data
            startActivity(emailIntent)
        }

        /**
         * Pokud je aplikace spuštěna poprvé po instalaci, nastaví se alarmy pro spuštění notifikací.
         *
         * Z třídy SharedPreferences je načtena boolean hodnota prvního spuštění,
         * která je po prvním spuštění true, poté ihned nastavena na false.
         */
        val prefs = getSharedPreferences("SharedPreferences", Context.MODE_PRIVATE)
        if (prefs.getBoolean("firstRun", true)) {

            AlrManager().setAlarm(this)
            prefs.edit().putBoolean("firstRun", false).apply()
        }

    }
}
