package cz.czu.pef.spaceboy

import android.Manifest.permission.ACCESS_COARSE_LOCATION
import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.PendingIntent
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager.PERMISSION_GRANTED
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Color
import android.os.Build
import androidx.core.app.ActivityCompat
import androidx.core.app.NotificationCompat
import androidx.core.app.NotificationCompat.BigTextStyle

@Suppress("DEPRECATION")
class NotificationReceiver: BroadcastReceiver() {

    lateinit var notificationManager: NotificationManager
    lateinit var notificationChannel: NotificationChannel
    lateinit var builder: NotificationCompat.Builder
    private val channelId = "cz.czu.pef.spaceboy"
    private val description = "Notification"

    override fun onReceive(context: Context, intent: Intent) {

        notificationManager = context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager

        val type = intent.getIntExtra("type", 0)

        /**
         * Ochrana před pádem aplikace v případě nedoručení Extra skrz Intent.
         */
        if (type == 1 || type == 2 ||
            (type == 3 && ActivityCompat.checkSelfPermission(context,
                ACCESS_COARSE_LOCATION) == PERMISSION_GRANTED)) {
            var contentTitle = ""
            var contentText = ""
            var className = ""
            var largeIcon: Bitmap? = null

            /**
             * Na základě typu alarmu, který dorazil přes Extra v Intentu, se zvolí,
             * jaký text a ikona se zobrazí v notifikaci a která aktivita se spustí po kliku
             * na notifikaci v notifikační liště.
             */
            when (type) {
                1 -> {
                    contentTitle = context.resources.getString(R.string.apod)
                    contentText = context.resources.getString(R.string.notif_apod)
                    className = "cz.czu.pef.spaceboy.ApodActivity"
                    largeIcon = BitmapFactory.decodeResource(context.resources, R.drawable.notif_apod)
                }
                2 -> {
                    contentTitle = context.resources.getString(R.string.asteroids)
                    contentText = context.resources.getString(R.string.notif_asteroid)
                    className = "cz.czu.pef.spaceboy.AsteroidsActivity"
                    largeIcon = BitmapFactory.decodeResource(context.resources, R.drawable.notif_asteroids)
                }
                3 -> {
                    contentTitle = context.resources.getString(R.string.iss_passes)
                    contentText = context.resources.getString(R.string.notif_iss)
                    className = "cz.czu.pef.spaceboy.IssPassesActivity"
                    largeIcon = BitmapFactory.decodeResource(context.resources, R.drawable.notif_iss)
                }
            }

            /**
             * Proces vytvoření notifikace
             */
            val pendingIntent = PendingIntent.getActivity(
                context,
                0,
                Intent(context, Class.forName(className)),
                PendingIntent.FLAG_UPDATE_CURRENT)

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                notificationChannel = NotificationChannel(
                    channelId,
                    description,
                    NotificationManager.IMPORTANCE_HIGH)
                notificationManager.createNotificationChannel(notificationChannel)

                builder = NotificationCompat.Builder(context, channelId)
                    .setContentTitle(contentTitle)
                    .setContentText(contentText)
                    .setSmallIcon(R.drawable.ic_stat_earth)
                    .setLargeIcon(largeIcon!!)
                    .setColor(Color.argb(100, 11, 61, 145))
                    .setContentIntent(pendingIntent)
                    .setAutoCancel(true)
                    .setStyle(BigTextStyle().bigText(contentText))
            }
            else {
                builder = NotificationCompat.Builder(context)
                    .setContentTitle(contentTitle)
                    .setContentText(contentText)
                    .setSmallIcon(R.drawable.ic_stat_earth)
                    .setLargeIcon(largeIcon!!)
                    .setColor(Color.argb(100, 11, 61, 145))
                    .setContentIntent(pendingIntent)
                    .setAutoCancel(true)
                    .setStyle(BigTextStyle().bigText(contentText))
            }

            notificationManager.notify(type, builder.build())
        }
    }
}