package cz.czu.pef.spaceboy

import android.Manifest.permission.ACCESS_COARSE_LOCATION
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager.PERMISSION_GRANTED
import android.net.Uri
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.view.View
import androidx.core.app.ActivityCompat
import kotlinx.android.synthetic.main.activity_settings.*

class SettingsActivity : AppCompatActivity() {

    private val urlCzu = "https://www.czu.cz/cs/"
    private val urlNasa = "https://api.nasa.gov/"
    private val urlN2yo = "https://www.n2yo.com/api/"
    private val urlOpenNotify = "http://api.open-notify.org/"
    private val urlFreepik = "https://www.freepik.com/"
    private val urlFlaticon = "https://www.flaticon.com/"

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_settings)

        /**
         * Načtení nastavení přepínačů ze SharedPreferences
         */
        val prefs = getSharedPreferences("SharedPreferences", Context.MODE_PRIVATE)
        val perm = ActivityCompat.checkSelfPermission(this,
            ACCESS_COARSE_LOCATION) == PERMISSION_GRANTED

        switch_notifApod.isChecked = prefs.getBoolean("notifApod", true)
        switch_notifAsteroids.isChecked = prefs.getBoolean("notifAsteroids", true)
        switch_notifIss.isChecked = prefs.getBoolean("notifIss", true)

        if (switch_notifIss.isChecked && !perm) {
            text_permissionNeeded.visibility = View.VISIBLE }

        /**
         * Akce při změně přepínače
         * isChecked: true -> spustí se alarm na pozadí a nastavení přepínače je uloženo
         * isChecked: false -> zruší se alarm a nastavení přepínače je uloženo
         */
        switch_notifApod.setOnCheckedChangeListener {_, isChecked ->
            if (isChecked) {
                prefs.edit().putBoolean("notifApod", true).apply()
                AlrManager().setAlarmSingle(this, "notifApod", 1)
            }
            else {
                prefs.edit().putBoolean("notifApod", false).apply()
                AlrManager().setAlarmSingle(this, "notifApod", 1)
            }
        }

        switch_notifAsteroids.setOnCheckedChangeListener {_, isChecked ->
            if (isChecked) {
                prefs.edit().putBoolean("notifAsteroids", true).apply()
                AlrManager().setAlarmSingle(this, "notifAsteroids", 2)
            }
            else {
                prefs.edit().putBoolean("notifAsteroids", false).apply()
                AlrManager().setAlarmSingle(this, "notifAsteroids", 2)
            }
        }

        switch_notifIss.setOnCheckedChangeListener {_, isChecked ->
            if (isChecked) {
                prefs.edit().putBoolean("notifIss", true).apply()
                if (!perm) {text_permissionNeeded.visibility = View.VISIBLE}
                AlrManager().setAlarmSingle(this, "notifIss", 3)
            }
            else {
                prefs.edit().putBoolean("notifIss", false).apply()
                text_permissionNeeded.visibility = View.GONE
                AlrManager().setAlarmSingle(this, "notifIss", 3)
            }
        }

        /**
         * Nastavení prokliku na weby
         */
        image_logo_czu.setOnClickListener {
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(urlCzu)))
        }

        image_nasa_api.setOnClickListener {
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(urlNasa)))
        }

        image_logo_n2yo.setOnClickListener {
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(urlN2yo)))
        }

        image_open_notify.setOnClickListener {
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(urlOpenNotify)))
        }

        textView_freepik.setOnClickListener {
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(urlFreepik)))
        }

        textView_flaticon.setOnClickListener {
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(urlFlaticon)))
        }
    }
}
